@@ -27,7 +27,6 @@
 #include <js/Value.h>
 #include <js/experimental/TypedData.h>
 #include <jsapi.h>        // for InformalValueTypeName, JS_TypeOfValue
-#include <jsfriendapi.h>  // for JS_GetObjectFunction
 #include <jspubtd.h>      // for JSTYPE_FUNCTION
 
 #include "gi/arg-cache.h"
@@ -217,31 +216,30 @@ struct BaseInfo {
 };
 
 // boxed / union / GObject
-struct GTypedType {
-    explicit GTypedType(GType gtype) : m_gtype(gtype) {}
-    constexpr GType gtype() const { return m_gtype; }
-
- protected:
-    GType m_gtype;
-};
-
-struct RegisteredType : GTypedType {
+struct RegisteredType {
     RegisteredType(GType gtype, GIInfoType info_type)
-        : GTypedType(gtype), m_info_type(info_type) {}
+        : m_gtype(gtype), m_info_type(info_type) {}
     explicit RegisteredType(GIBaseInfo* info)
-        : GTypedType(g_registered_type_info_get_g_type(info)),
+        : m_gtype(g_registered_type_info_get_g_type(info)),
           m_info_type(g_base_info_get_type(info)) {
         g_assert(m_gtype != G_TYPE_NONE &&
                  "Use RegisteredInterface for this type");
     }
 
+    constexpr GType gtype() const { return m_gtype; }
+
+    GType m_gtype;
     GIInfoType m_info_type : 5;
 };
 
-struct RegisteredInterface : BaseInfo, GTypedType {
+struct RegisteredInterface : BaseInfo {
     explicit RegisteredInterface(GIBaseInfo* info)
         : BaseInfo(info, GjsAutoTakeOwnership{}),
-          GTypedType(g_registered_type_info_get_g_type(m_info)) {}
+          m_gtype(g_registered_type_info_get_g_type(m_info)) {}
+
+    constexpr GType gtype() const { return m_gtype; }
+
+    GType m_gtype;
 };
 
 struct Callback : Nullable, BaseInfo {
@@ -576,8 +574,6 @@ struct GBytesIn : BoxedIn {
     using BoxedIn::BoxedIn;
     bool in(JSContext*, GjsFunctionCallState*, GIArgument*,
             JS::HandleValue) override;
-    bool release(JSContext* cx, GjsFunctionCallState* state, GIArgument* in_arg,
-                 GIArgument* out_arg) override;
 };
 
 struct GBytesInTransferNone : GBytesIn {
@@ -717,12 +713,6 @@ struct CallerAllocatesOut : GenericOut, CallerAllocates {
     }
 };
 
-struct BoxedCallerAllocatesOut : CallerAllocatesOut, GTypedType {
-    using GTypedType::GTypedType;
-    bool release(JSContext*, GjsFunctionCallState*, GIArgument*,
-                 GIArgument*) override;
-};
-
 GJS_JSAPI_RETURN_CONVENTION
 bool NotIntrospectable::in(JSContext* cx, GjsFunctionCallState* state,
                            GIArgument*, JS::HandleValue) {
@@ -853,10 +843,10 @@ bool CallbackIn::in(JSContext* cx, GjsFunctionCallState* state, GIArgument* arg,
             return false;
         }
 
-        JS::RootedFunction func(cx, JS_GetObjectFunction(&value.toObject()));
+        JS::RootedObject callable(cx, &value.toObject());
         bool is_object_method = !!state->instance_object;
-        trampoline = GjsCallbackTrampoline::create(cx, func, m_info, m_scope,
-                                                   is_object_method, false);
+        trampoline = GjsCallbackTrampoline::create(
+            cx, callable, m_info, m_scope, is_object_method, false);
         if (!trampoline)
             return false;
         if (m_scope == GI_SCOPE_TYPE_NOTIFIED && is_object_method) {
@@ -1091,7 +1081,9 @@ bool FlagsIn::in(JSContext* cx, GjsFunctionCallState*, GIArgument* arg,
         return false;
 
     if ((uint64_t(number) & m_mask) != uint64_t(number)) {
-        gjs_throw(cx, "%" PRId64 " is not a valid value for flags argument %s",
+        gjs_throw(cx,
+                  "0x%" G_GINT64_MODIFIER
+                  "x is not a valid value for flags argument %s",
                   number, m_arg_name);
         return false;
     }
@@ -1188,8 +1180,8 @@ bool GClosureInTransferNone::in(JSContext* cx, GjsFunctionCallState* state,
         return report_typeof_mismatch(cx, m_arg_name, value,
                                       ExpectedType::FUNCTION);
 
-    JS::RootedFunction func(cx, JS_GetObjectFunction(&value.toObject()));
-    GClosure* closure = Gjs::Closure::create_marshaled(cx, func, "boxed");
+    JS::RootedObject callable(cx, &value.toObject());
+    GClosure* closure = Gjs::Closure::create_marshaled(cx, callable, "boxed");
     gjs_arg_set(arg, closure);
     g_closure_ref(closure);
     g_closure_sink(closure);
@@ -1208,7 +1200,6 @@ bool GBytesIn::in(JSContext* cx, GjsFunctionCallState* state, GIArgument* arg,
 
     JS::RootedObject object(cx, &value.toObject());
     if (JS_IsUint8Array(object)) {
-        state->ignore_release.insert(arg);
         gjs_arg_set(arg, gjs_byte_array_get_bytes(object));
         return true;
     }
@@ -1219,15 +1210,6 @@ bool GBytesIn::in(JSContext* cx, GjsFunctionCallState* state, GIArgument* arg,
         cx, object, arg, GI_DIRECTION_IN, GI_TRANSFER_EVERYTHING, G_TYPE_BYTES);
 }
 
-GJS_JSAPI_RETURN_CONVENTION
-bool GBytesIn::release(JSContext* cx, GjsFunctionCallState* state,
-                       GIArgument* in_arg, GIArgument* out_arg) {
-    if (state->ignore_release.erase(in_arg))
-        return BoxedIn::release(cx, state, in_arg, out_arg);
-
-    return BoxedInTransferNone::release(cx, state, in_arg, out_arg);
-}
-
 GJS_JSAPI_RETURN_CONVENTION
 bool InterfaceIn::in(JSContext* cx, GjsFunctionCallState* state,
                      GIArgument* arg, JS::HandleValue value) {
@@ -1348,8 +1330,7 @@ bool ExplicitArrayInOut::out(JSContext* cx, GjsFunctionCallState* state,
     GIArgument* length_arg = &(state->out_cvalue(m_length_pos));
     size_t length = gjs_g_argument_get_array_length(m_tag, length_arg);
 
-    return gjs_value_from_explicit_array(cx, value, &m_type_info, m_transfer,
-                                         arg, length);
+    return gjs_value_from_explicit_array(cx, value, &m_type_info, arg, length);
 }
 
 GJS_JSAPI_RETURN_CONVENTION
@@ -1436,13 +1417,6 @@ bool CallerAllocatesOut::release(JSContext*, GjsFunctionCallState*,
     return true;
 }
 
-GJS_JSAPI_RETURN_CONVENTION
-bool BoxedCallerAllocatesOut::release(JSContext*, GjsFunctionCallState*,
-                                      GIArgument* in_arg, GIArgument*) {
-    g_boxed_free(m_gtype, gjs_arg_steal<void*>(in_arg));
-    return true;
-}
-
 GJS_JSAPI_RETURN_CONVENTION
 bool CallbackIn::release(JSContext*, GjsFunctionCallState*, GIArgument* in_arg,
                          GIArgument* out_arg [[maybe_unused]]) {
@@ -1590,8 +1564,7 @@ std::unique_ptr<T> Argument::make(uint8_t index, const char* name,
 }
 
 //  Needed for unique_ptr with incomplete type
-// COMPAT: in C++20, use default initializers for these bitfields in the header
-ArgsCache::ArgsCache() : m_is_method(false), m_has_return(false) {}
+ArgsCache::ArgsCache() = default;
 ArgsCache::~ArgsCache() = default;
 
 bool ArgsCache::initialize(JSContext* cx, GICallableInfo* callable) {
@@ -2127,6 +2100,21 @@ void ArgsCache::build_instance(GICallableInfo* callable) {
         GjsArgumentFlags::NONE);
 }
 
+static size_t get_type_info_interface_size(GITypeInfo* type_info) {
+    GjsAutoBaseInfo interface_info = g_type_info_get_interface(type_info);
+    g_assert(interface_info);
+
+    GIInfoType interface_type = g_base_info_get_type(interface_info);
+
+    if (interface_type == GI_INFO_TYPE_STRUCT) {
+        return g_struct_info_get_size(interface_info);
+    } else if (interface_type == GI_INFO_TYPE_UNION) {
+        return g_union_info_get_size(interface_info);
+    }
+
+    return 0;
+}
+
 void ArgsCache::build_arg(uint8_t gi_index, GIDirection direction,
                           GIArgInfo* arg, GICallableInfo* callable,
                           bool* inc_counter_out) {
@@ -2155,38 +2143,43 @@ void ArgsCache::build_arg(uint8_t gi_index, GIDirection direction,
     GITypeTag type_tag = g_type_info_get_tag(&type_info);
     if (direction == GI_DIRECTION_OUT &&
         (flags & GjsArgumentFlags::CALLER_ALLOCATES)) {
-        if (type_tag != GI_TYPE_TAG_INTERFACE) {
-            set_argument_auto<Arg::NotIntrospectable>(
-                common_args, OUT_CALLER_ALLOCATES_NON_STRUCT);
-            return;
-        }
+        size_t size = 0;
 
-        GjsAutoBaseInfo interface_info = g_type_info_get_interface(&type_info);
-        g_assert(interface_info);
+        if (type_tag == GI_TYPE_TAG_ARRAY) {
+            GIArrayType array_type = g_type_info_get_array_type(&type_info);
 
-        GIInfoType interface_type = g_base_info_get_type(interface_info);
+            switch (array_type) {
+                case GI_ARRAY_TYPE_C: {
+                    GjsAutoTypeInfo param_info;
+                    int n_elements =
+                        g_type_info_get_array_fixed_size(&type_info);
 
-        size_t size;
-        if (interface_type == GI_INFO_TYPE_STRUCT) {
-            size = g_struct_info_get_size(interface_info);
-        } else if (interface_type == GI_INFO_TYPE_UNION) {
-            size = g_union_info_get_size(interface_info);
-        } else {
-            set_argument_auto<Arg::NotIntrospectable>(
-                common_args, OUT_CALLER_ALLOCATES_NON_STRUCT);
-            return;
-        }
+                    if (n_elements <= 0)
+                        break;
 
-        if (type_tag == GI_TYPE_TAG_INTERFACE) {
-            GjsAutoBaseInfo interface_info =
-                g_type_info_get_interface(&type_info);
-            GType gtype = g_registered_type_info_get_g_type(interface_info);
-            if (g_type_is_a(gtype, G_TYPE_BOXED)) {
-                auto* gjs_arg = set_argument_auto<Arg::BoxedCallerAllocatesOut>(
-                    common_args, gtype);
-                gjs_arg->m_allocates_size = size;
-                return;
+                    param_info = g_type_info_get_param_type(&type_info, 0);
+                    GITypeTag param_tag = g_type_info_get_tag(param_info);
+
+                    if (param_tag == GI_TYPE_TAG_INTERFACE) {
+                        size = get_type_info_interface_size(param_info);
+                    } else {
+                        size = gjs_array_get_element_size(param_tag);
+                    }
+
+                    size *= n_elements;
+                    break;
+                }
+                default:
+                    break;
             }
+        } else if (type_tag == GI_TYPE_TAG_INTERFACE) {
+            size = get_type_info_interface_size(&type_info);
+        }
+
+        if (!size) {
+            set_argument_auto<Arg::NotIntrospectable>(
+                common_args, OUT_CALLER_ALLOCATES_NON_STRUCT);
+            return;
         }
 
         auto* gjs_arg = set_argument_auto<Arg::CallerAllocatesOut>(common_args);
