@@ -17,6 +17,7 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Utility.h>  // for UniqueChars
+#include <js/Value.h>
 #include <jsapi.h>       // for InformalValueTypeName, JS_NewObjectWithGivenP...
 #include <mozilla/HashTable.h>
 
@@ -25,7 +26,6 @@
 #include "gi/function.h"
 #include "gi/fundamental.h"
 #include "gi/repo.h"
-#include "gi/value.h"
 #include "gi/wrapperutils.h"
 #include "gjs/atoms.h"
 #include "gjs/context-private.h"
@@ -483,11 +483,11 @@ bool FundamentalBase::to_gvalue(JSContext* cx, JS::HandleObject obj,
             return true;
         } else if (g_value_type_transformable(instance->gtype(),
                                               G_VALUE_TYPE(gvalue))) {
-            Gjs::AutoGValue instance_value;
+            GValue instance_value = {0};
             g_value_init(&instance_value, instance->gtype());
             g_value_set_instance(&instance_value, instance->m_ptr);
-            if (g_value_transform(&instance_value, gvalue))
-                return true;
+            g_value_transform(&instance_value, gvalue);
+            return true;
         }
 
         gjs_throw(cx,
