@@ -814,10 +814,8 @@ gjstest_test_func_util_misc_strv_concat_pointers(void)
 static void
 gjstest_test_profiler_start_stop(void)
 {
-    GjsAutoUnref<GjsContext> context =
-        static_cast<GjsContext *>(g_object_new(GJS_TYPE_CONTEXT,
-                                               "profiler-enabled", TRUE,
-                                               nullptr));
+    GjsAutoUnref<GjsContext> context = GJS_CONTEXT(
+        g_object_new(GJS_TYPE_CONTEXT, "profiler-enabled", TRUE, nullptr));
     GjsProfiler *profiler = gjs_context_get_profiler(context);
 
     gjs_profiler_set_filename(profiler, "dont-conflict-with-other-test.syscap");
@@ -1014,20 +1012,20 @@ static void gjstest_test_args_rounded_values() {
 
 static void gjstest_test_func_gjs_context_argv_array() {
     GjsAutoUnref<GjsContext> gjs = gjs_context_new();
-    GjsAutoError error;
+    GError* error = NULL;
     int status;
 
     const char* argv[1] = {"test"};
-    bool ok =
-        gjs_context_define_string_array(gjs, "ARGV", 1, argv, error.out());
+    bool ok = gjs_context_define_string_array(gjs, "ARGV", 1, argv, &error);
 
     g_assert_no_error(error);
+    g_clear_error(&error);
     g_assert_true(ok);
 
     ok = gjs_context_eval(gjs, R"js(
         imports.system.exit(ARGV[0] === "test" ? 0 : 1)
     )js",
-                          -1, "<main>", &status, error.out());
+                          -1, "<main>", &status, &error);
 
     g_assert_cmpint(status, ==, 0);
     g_assert_error(error, GJS_ERROR, GJS_ERROR_SYSTEM_EXIT);
